' 
' SIDcog TrainSound 
'
' Set up to run on demo board.  Use mouse to control whistle, bell, and train speed.
'  
'FIXME: need to improve bell sound
'FIXME: get mouse volume control to work better
'FIXME: get bell toggle control to work better (right mouse button)
'FIXME: save defaults in high eeprom (volume, train speed)


CON _clkmode = xtal1 + pll16x
    _xinfreq = 5_000_000    
    
    'software constants
    freqAdjust  = 1.43                       ' Adjust the relative frequency of both 1 and 2 
    freq1       = trunc(6000.0*freqAdjust)   ' Whistle frequency 1                              
    freq2       = trunc(4750.0*freqAdjust)   ' Whistle frequency 2
    sleep       = 10000*60*4                 'goto sleep after ~4 mins
    climb_rate  = 20                         'whistle pitch climb rate (how fast pitch increases as button held down)
    fall_rate   = 40                         'whistle pitch fall (controls persistence of whistle sound after button release)
    'hardware constants
    spkr_l      = 10
    spkr_r      = 11
    tv_base_pin = 12
    mouse_dat   = 24
    mouse_clk   = 25
    kb_dat      = 26
    kb_clk      = 27

    
var
    long volume, updaterate, whistle, bell, frame, finetune, whistle_decay


OBJ
  SID  : "SIDCog"
  mouse: "mouse"
  kb   : "keyboard"
  term : "tv_terminal"


PUB Main | tmp, junk, timer

  SID.start(spkr_l,spkr_r)               'Start SID in cog 1 (supposedly better sound quality per Jon/Chip) 
  term.start(tv_base_pin)
  mouse.start(mouse_dat, mouse_clk)
  kb.start(kb_dat, kb_clk)

  'defaults
  volume := 5                             'sound volume
  updateRate  := 130                      'Tuff tuff speed
  timer := 0
  SID.setVolume(volume)                   
  SID.setCutoff(150)
  SID.setResonance(15)

  
  repeat
    waitcnt(cnt+(clkfreq/updateRate))
    frame++

    'get input
    doMouse
    doKeyboard
    
    'shut train sound off after ~4 minutes inactivity
    timer += (10000/updaterate)
    if timer >  sleep
      SID.setVolume(0)
      repeat until mouse.buttons & %11        'press mouse button to wake up
      SID.setVolume(volume)
    if whistle or tmp
      timer := 0 

    'Tuff tuff sound
    if updaterate>5
      doChuff

    'bell sound
    if bell and not whistle
      doBell
       
    'Whistle sound
    doWhistle

    'tv display
    if (frame&15)==7 and updaterate>5
      if junk//2==0
        term.str(string("CHUFF...     "))
      else
        term.str(string("     ...CHUFF"))
      junk++
      if whistle
        term.str(string(" WHISTLE"))
      else
        term.str(string("        "))
      if bell
        term.str(string(" BELL"))
      else
        term.str(string("     "))
      term.str(string(13))
    

   
pri doMouse|tmp
'process mouse inputs

    whistle := mouse.buttons & %1             'Left mouse button for whistle   
    if mouse.buttons & %10                    'Right mouse button toggle bell on/off
       not bell
    tmp := updateRate + mouse.delta_z*3       'scroll wheel changes chuff-chuff speed
    
    if mouse.buttons & %10                     'hold down right mouse & scroll to adjust volume
      if mouse.delta_z>0
        volume++
      elseif mouse.delta_z<0
        volume--
    else       
      updateRate := 4 #> tmp <# 230            'scroll wheel for controlling train speed


pri doKeyboard|tmp
'process keyboard inputs, do bounds checking

    tmp := kb.key
    case tmp
       194 : volume++                          'up arrow for more volume
       195 : volume--                          'dn arrow for less volume
       32  : whistle:=1                        'space bar whistle

    volume := 0 #> volume <# 12 


pri doChuff|tmp
'chuff-chuff sound

    if (frame&15)==7 and not whistle
      SID.play(0,5000+((?tmp)&511),SID#NOISE,5,5,5,9)
    if (frame&15) == 12 and not whistle
      SID.noteOff(0)
     

pri doBell
'bell sound

    if cnt & (%1 <- 25)                         'toggle bell sound w/ cnt bit 25 (~1/sec @ 80MHz)
      SID.play(2,freq2word(400),SID#TRIANGLE,0, 0, 6, 9)
    SID.noteOff(2)
     

pri doWhistle  
'whistle sound

    if whistle                                           'whistle climb
      SID.setVolume(volume+1)
      finetune:=(finetune+climb_rate)<#1000
      SID.setFilterMask(true,true,true) 
      SID.setFilterType(false,true,true)
      SID.play(0,17_000,SID#NOISE,0,0,3,3)
      SID.play(1,freq1+finetune,SID#TRIANGLE,1,1,4,3) 
      SID.play(2,freq2+finetune,SID#TRIANGLE,1,1,6,3) 
      whistle_decay:=finetune
    elseif whistle_decay                                 ' whistle fall
      finetune:= 0 #> (finetune-fall_rate)
      SID.setFilterMask(true,true,true) 
      SID.setFilterType(false,true,true)
      SID.play(0,17_000,SID#NOISE,0,0,3,3)
      SID.play(1,freq1+finetune,SID#TRIANGLE,1,1,4,3) 
      SID.play(2,freq2+finetune,SID#TRIANGLE,1,1,6,3) 
      whistle_decay:=finetune                             
    else                                                  'whistle off
      SID.setVolume(volume)
      finetune:=0
      SID.setFilterMask(true,false,false)
      SID.setFilterType(true,true,false)
      SID.noteOff(1)
      SID.noteOff(2)


pri freq2word(freq)|tmp
'converts frequency (Hz) to 16-bit value

tmp := 65536/3900*freq
tmp := 0#> tmp <#65536
return tmp  
               